/*
 * gbiff2 program
 * Copyright (C) 2000-2003 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <stdlib.h>
#include <string.h>
#include <popt.h>
#include <libintl.h>

#include "gbiff2.h"
#include "interface.h"
#include "support.h"
#include "callbacks.h"


/********************************************************************************
 * Function: main 
 * Input   : argc, argv
 * Output  : /
 * Return  : Error code (0 if OK, 1 else)
 ********************************************************************************/
int main (int argc, char **argv) {
  Biff biff = {g_strdup (DEFAULT_TITLE),	/* title		*/
	       DEFAULT_PROTOCOL,		/* protocol		*/
	       g_strdup (""),			/* address		*/
	       g_strdup (""),			/* user			*/
	       g_strdup (""),			/* password		*/
	       DEFAULT_POLLTIME,		/* polltime		*/
	       DEFAULT_POPTIME,			/* poptime		*/

	       g_strdup (DEFAULT_POPUP_GEOMETRY),/*popupGeometry	*/
	       DEFAULT_POPUP_DECORATED,		/* popupDecorated	*/
	       DEFAULT_POPUP_SMART_GEOMETRY,	/* popupSmartGeometry	*/
	       g_strdup (DEFAULT_BIFF_GEOMETRY),/* biffGeometry		*/
	       DEFAULT_BIFF_DECORATED,		/* biffDecorated	*/
	       DEFAULT_BIFF_SMART_GEOMETRY,	/* biffSmartGeometry	*/

	       DEFAULT_DISPLAY_POPUP,		/* displayPopup		*/
	       DEFAULT_DISPLAY_TITLE,		/* displayTitle		*/
	       DEFAULT_DISPLAY_DATE,		/* displayDate		*/
	       DEFAULT_DISPLAY_READMAIL,	/* displayReadMail	*/
	       DEFAULT_FROM_SIZE,		/* fromSize		*/
	       DEFAULT_SUBJECT_SIZE,		/* subjectSize		*/
	       DEFAULT_SOUND_TYPE,		/* soundType		*/
	       DEFAULT_SOUND_VOLUME,		/* soundVolume		*/

	       g_strdup (DEFAULT_MAILAPP),	/* mailApp		*/
	       g_strdup (DEFAULT_SOUNDFILE),	/* soundFile		*/
	       g_strdup (DEFAULT_TITLE_FONT),	/* titleFont		*/
	       g_strdup (DEFAULT_BODY_FONT),	/* bodyFont		*/
	       g_strdup (DEFAULT_BIFF_FONT),	/* biffFont		*/
	       g_strdup (DEFAULT_READ_COLOR),	/* readColor		*/
	       g_strdup (DEFAULT_UNREAD_COLOR),	/* unreadColor		*/
	       g_strdup (DEFAULT_MAIL_IMAGE_FILE),   /* mailImageFile	*/
	       g_strdup (DEFAULT_NOMAIL_IMAGE_FILE), /* nomailImageFile	*/
	       
	       g_strdup (DEFAULT_GLADE_FILE),	/* gladeFile		*/
	       g_strdup (""),			/* configFile		*/

	       {0},				/* headers		*/
	       0,				/* headerSize		*/
	       0,				/* unread		*/
	       0,				/* read			*/
	       0,				/* status		*/

	       AUTOMATIC_MODE,			/* pollMode		*/
	       0,				/* polltag		*/
	       0,				/* poptag		*/

	       0,				/* noconfigure		*/
	       0,				/* debug		*/
	       0,				/* block		*/
	       0,				/* action		*/

	       0,				/* xml			*/
	       0,				/* biff			*/
	       0,				/* popup		*/
	       0};				/* popupTreeView	*/


/*********************/
/*   GNOME Version   */
/*********************/
#ifdef USE_GNOME
  static gboolean gbiff2_applet_factory (PanelApplet *applet, const gchar *iid, gpointer data) {
    gboolean retval = FALSE;
    if (!strcmp (iid, "OAFIID:GNOME_gbiff2Applet"))
      retval = buildApplet (GTK_WIDGET (applet), data);
    return retval;
  }

#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, GBIFF2_LOCALEDIR);
#   ifdef HAVE_BIND_TEXTDOMAIN_CODESET
      bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
#   endif
  textdomain (GETTEXT_PACKAGE);
#endif

# if defined(PREFIX) && defined(SYSCONFDIR) && defined(DATADIR) && defined(LIBDIR)
    gnome_program_init ("gbiff2", "0", LIBGNOMEUI_MODULE, argc, argv, GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
# else
    gnome_program_init ("gbiff2", "0", LIBGNOMEUI_MODULE, argc, argv, GNOME_PARAM_NONE);
# endif
  panel_applet_factory_main ("OAFIID:GNOME_gbiff2Applet_Factory", PANEL_TYPE_APPLET, gbiff2_applet_factory, &biff);


#else


/*********************/
/*    GTK Version    */
/*********************/
  poptContext poptcon;
  char *configFile = 0;
  int i, status;
  struct poptOption options[] = {
    {"config",     'c', POPT_ARG_STRING, &configFile,       0, _("gbiff2 config file to use"),      _("file")},
    {"noconfigure",'n', POPT_ARG_NONE,   &biff.noconfigure, 0, _("skip the configure process"),     NULL},
    {"debug",      'g', POPT_ARG_NONE,   &biff.debug,       0, _("debug output"),                   NULL},
    POPT_AUTOHELP
    {NULL, '\0', 0, NULL, 0, NULL, NULL}
  };

#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, GBIFF2_LOCALEDIR);
#    ifdef HAVE_BIND_TEXTDOMAIN_CODESET
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
#    endif
  textdomain (PACKAGE_NAME);
#endif

  poptcon = poptGetContext ("gbiff2", argc,  (const char **) argv, options, 0);
  while ((status = poptGetNextOpt(poptcon)) >= 0);
  if (status < -1) {
    fprintf (stderr, "%s: %s\n\n", poptBadOption(poptcon, POPT_BADOPTION_NOALIAS),  poptStrerror(status));
    poptPrintHelp (poptcon, stderr, 0);
    exit (1);
  }
  poptGetNextOpt(poptcon);

  /* Allocate headers in biff */
  for (i=0; i<MAX_HEADERS; i++)
    biff.headers[i] = g_malloc (sizeof (Header));

  /* Get the configuration file */
  if (configFile != NULL)
    biff.configFile = g_strdup (configFile);
  else
    biff.configFile = g_strconcat ((gchar *) g_getenv("HOME"), "/.gbiff2rc", NULL);

  /* Get default address */
  biff.address = g_strdup ((gchar *) g_getenv ("MAIL"));

  /* Get default username */
  biff.user = g_strdup ((gchar *) g_getenv ("LOGNAME"));

  /* Set the debug flag if DEBUG is defined */
#ifdef DEBUG
  biff.debug = 1;
#endif

  /* Load the config file */
  loadConfig (&biff);

  /* GTK inialisation */
  gtk_init (&argc, &argv);

  /* Interface creation */
  if (buildInterface (&biff) == 1)
    return 1;

  if (biff.noconfigure) 
    on_OkButton_clicked (0, &biff);
  else
    gtk_widget_show (__widget(&biff, "ConfigurationDialog"));

  /* GTK main loop */
  gtk_main ();

#endif /* USE_GNOME */
  /* Exit */
  return 0;
}

/********************************************************************************
 * Function: Load gbiff2 configuration
 * Input   : a biff structure
 * Output  : Save parameters to biff->configFile
 * Return  : Error code (0 if OK, 1 else)
 * Note    : /
 ********************************************************************************/
int loadConfig (Biff *biff) {
  char buffer[256];
  FILE *file;

  /* Open config file */
  if ((file = fopen (biff->configFile, "r")) == NULL) {
    g_warning (_("Cannot load config file (%s).\n"), biff->configFile);
    return 1;
  }

  /* Parse the config file */
  while (!feof (file)) {
    fgets (buffer, 255, file);
    if (((gchar *) (strstr (buffer, "Title"))) == buffer) {
      g_free (biff->title);
      biff->title = g_strdup (g_strstrip ((char *) (strchr (buffer, ':')+1)));
    }
    if (((gchar *) (strstr (buffer, "Protocol"))) == buffer) 
      sscanf ((char *) (strchr (buffer, ':')+1), "%d", &biff->protocol);
    if (((gchar *) (strstr (buffer, "Address"))) == buffer) {
      g_free (biff->address);
      biff->address = g_strdup (g_strstrip ((char *) (strchr (buffer, ':')+1)));
    }
    if (((gchar *) (strstr (buffer, "User"))) == buffer) {
      g_free (biff->user);
      biff->user = g_strdup (g_strstrip ((char *) (strchr (buffer, ':')+1)));
    }
    if (((gchar *) (strstr (buffer, "Polltime"))) == buffer)
      sscanf ((char *) (strchr (buffer, ':')+1), "%d", &biff->polltime);
    if (((gchar *) (strstr (buffer, "Poptime"))) == buffer)
      sscanf ((char *) (strchr (buffer, ':')+1), "%d", &biff->poptime);
    if (((gchar *) (strstr (buffer, "Popup geometry"))) == buffer) {
      g_free (biff->popupGeometry);
      biff->popupGeometry = g_strdup (g_strstrip ((char *) (strchr (buffer, ':')+1)));
    }
    if (((gchar *) (strstr (buffer, "Popup smart geometry"))) == buffer)
      sscanf ((char *) (strchr (buffer, ':')+1), "%d", &biff->popupSmartGeometry);
    if (((gchar *) (strstr (buffer, "Popup decoration"))) == buffer)
      sscanf ((char *) (strchr (buffer, ':')+1), "%d", &biff->popupDecorated);
    if (((gchar *) (strstr (buffer, "Biff geometry"))) == buffer) {
      g_free (biff->biffGeometry);
      biff->biffGeometry = g_strdup (g_strstrip ((char *) (strchr (buffer, ':')+1)));
    }
    if (((gchar *) (strstr (buffer, "Biff decoration"))) == buffer)
      sscanf ((char *) (strchr (buffer, ':')+1), "%d", &biff->biffDecorated);
    if (((gchar *) (strstr (buffer, "Biff smart geometry"))) == buffer)
      sscanf ((char *) (strchr (buffer, ':')+1), "%d", &biff->biffSmartGeometry);
    if (((gchar *) (strstr (buffer, "Display popup"))) == buffer)
      sscanf ((char *) (strchr (buffer, ':')+1), "%d", &biff->displayPopup);
    if (((gchar *) (strstr (buffer, "Display title"))) == buffer)
      sscanf ((char *) (strchr (buffer, ':')+1), "%d", &biff->displayTitle);
    if (((gchar *) (strstr (buffer, "Display date"))) == buffer)
      sscanf ((char *) (strchr (buffer, ':')+1), "%d", &biff->displayDate);
    if (((gchar *) (strstr (buffer, "Display read mail"))) == buffer)
      sscanf ((char *) (strchr (buffer, ':')+1), "%d", &biff->displayReadMail);
    if (((gchar *) (strstr (buffer, "'From' field size"))) == buffer)
      sscanf ((char *) (strchr (buffer, ':')+1), "%d", &biff->fromSize);
    if (((gchar *) (strstr (buffer, "'Subject' field size"))) == buffer)
      sscanf ((char *) (strchr (buffer, ':')+1), "%d", &biff->subjectSize);
    if (((gchar *) (strstr (buffer, "Sound type"))) == buffer)
      sscanf ((char *) (strchr (buffer, ':')+1), "%d", &biff->soundType);
    if (((gchar *) (strstr (buffer, "Sound volume"))) == buffer)
      sscanf ((char *) (strchr (buffer, ':')+1), "%d", &biff->soundVolume);
    if (((gchar *) (strstr (buffer, "Mail application"))) == buffer) {
      g_free (biff->mailApp);
      biff->mailApp = g_strdup (g_strstrip ((char *) (strchr (buffer, ':')+1)));
    }
    if (((gchar *) (strstr (buffer, "Sound file"))) == buffer) {
      g_free (biff->soundFile);
      biff->soundFile = g_strdup (g_strstrip ((char *) (strchr (buffer, ':')+1)));
    }
    if (((gchar *) (strstr (buffer, "Popup title font"))) == buffer) {
      g_free (biff->titleFont);
      biff->titleFont = g_strdup (g_strstrip ((char *) (strchr (buffer, ':')+1)));
    }
    if (((gchar *) (strstr (buffer, "Popup body font"))) == buffer) { 
      g_free (biff->bodyFont);
      biff->bodyFont = g_strdup (g_strstrip ((char *) (strchr (buffer, ':')+1)));
    }
    if (((gchar *) (strstr (buffer, "Biff font"))) == buffer) {
      g_free (biff->biffFont);
      biff->biffFont = g_strdup (g_strstrip ((char *) (strchr (buffer, ':')+1)));
    }
    if (((gchar *) (strstr (buffer, "Read color"))) == buffer) {
      g_free (biff->readColor);
      biff->readColor = g_strdup (g_strstrip ((char *) (strchr (buffer, ':')+1)));
    }
    if (((gchar *) (strstr (buffer, "Unread color"))) == buffer) {
      g_free (biff->unreadColor );
      biff->unreadColor = g_strdup (g_strstrip ((char *) (strchr (buffer, ':')+1)));
    }
    if (((gchar *) (strstr (buffer, "Mail image"))) == buffer) {
      g_free (biff->mailImageFile);
      biff->mailImageFile = g_strdup (g_strstrip ((char *) (strchr (buffer, ':')+1)));
    }
    if (((gchar *) (strstr (buffer, "No mail image"))) == buffer) {
      g_free (biff->nomailImageFile);
      biff->nomailImageFile = g_strdup (g_strstrip ((char *) (strchr (buffer, ':')+1)));
    }
    if (((gchar *) (strstr (buffer, "Glade file"))) == buffer) {
      g_free (biff->gladeFile);
      biff->gladeFile = g_strdup (g_strstrip ((char *) (strchr (buffer, ':')+1)));
    }
  };

  /* Close config file */
  if (fclose (file) != 0) {
    g_warning (_("Problem while closing config file (%s).\n"), biff->configFile);
    return 1;
  }

  return 0;
}


/********************************************************************************
 * Function: Save gibff2 configuration
 * Input   : a biff structure
 * Output  : Save parameters to biff->configFile
 * Return  : Error code (0 if OK, 1 else)
 * Note    : /
 ********************************************************************************/
int saveConfig (Biff *biff) {
  FILE *file;

  /* Open config file */
  if ((file = fopen (biff->configFile, "w")) == NULL) {
    g_warning ("Cannot save config file (%s).\n", biff->configFile);
    return 1;
  }

  fprintf (file, "Title               :\t%s\n", biff->title);
  fprintf (file, "Protocol            :\t%d\n", biff->protocol);
  fprintf (file, "Address             :\t%s\n", biff->address);
  fprintf (file, "User                :\t%s\n", biff->user);
  fprintf (file, "Polltime            :\t%d\n", biff->polltime);
  fprintf (file, "Poptime             :\t%d\n", biff->poptime);
  fprintf (file, "Popup geometry      :\t%s\n", biff->popupGeometry);
  fprintf (file, "Popup decoration    :\t%d\n", biff->popupDecorated);
  fprintf (file, "Popup smart geometry:\t%d\n", biff->popupSmartGeometry);
  fprintf (file, "Biff geometry       :\t%s\n", biff->biffGeometry);
  fprintf (file, "Biff decoration     :\t%d\n", biff->biffDecorated);
  fprintf (file, "Biff smart geometry :\t%d\n", biff->biffSmartGeometry);
  fprintf (file, "Display popup       :\t%d\n", biff->displayPopup);
  fprintf (file, "Display title       :\t%d\n", biff->displayTitle);
  fprintf (file, "Display date        :\t%d\n", biff->displayDate);
  fprintf (file, "Display read mail   :\t%d\n", biff->displayReadMail);
  fprintf (file, "'From' field size   :\t%d\n", biff->fromSize);
  fprintf (file, "'Subject' field size:\t%d\n", biff->subjectSize);
  fprintf (file, "Sound type          :\t%d\n", biff->soundType);
  fprintf (file, "Sound volume        :\t%d\n", biff->soundVolume);
  fprintf (file, "Mail application    :\t%s\n", biff->mailApp);
  fprintf (file, "Sound file          :\t%s\n", biff->soundFile);
  fprintf (file, "Popup title font    :\t%s\n", biff->titleFont);
  fprintf (file, "Popup body font     :\t%s\n", biff->bodyFont);
  fprintf (file, "Biff font           :\t%s\n", biff->biffFont);
  fprintf (file, "Read color          :\t%s\n", biff->readColor);
  fprintf (file, "Unread color        :\t%s\n", biff->unreadColor);
  fprintf (file, "Mail image          :\t%s\n", biff->mailImageFile);
  fprintf (file, "No mail image       :\t%s\n", biff->nomailImageFile);
  fprintf (file, "Glade file          :\t%s\n", biff->gladeFile);

  /* Close config file */
  if (fclose (file) != 0) {
    g_warning (_("Problem while closing config file (%s).\n"), biff->configFile);
    return 1;
  }

  return 0;
}
